#!/usr/bin/python
import sys
from subprocess import *
from optparse import OptionParser
import math

def fairness(v):
	Num = len(v);
	Sum = sum(v) #0
	SumSq =	sum([x**2 for x in v]) #0	

	if Num * SumSq > 0:	
		return Sum**2 / (Num * SumSq);
	else:
		return None	

def toNumber(s):
    try:
        float(s)
        return float(s)	#True
    except ValueError:
        return s		#False

class Metrics:
	def __init__(self, cap=10e6, ntcp = 5, time = 60):
		self.ntcp = ntcp
		self.cap = cap
		self.time = time
		self.m = []
		self.mcw = []

		self.Efficiency = 0
		self.Breakdown = 0
		self.Fairness = []	#all,TCP,CC
		self.Droprate = []	#all,TCP,CC
		self.CwndAvg = []	#TCP,CC		
		self.QueueAvg = 0
		
	def load(self, data):
		for line in data:
			self.m.append(map(toNumber,line.split()))

	def calculate(self):
		# sum column of a "matrix" using array (normal format)
		# sum(m[i][1] for i in range(10)) see http://adorio-research.org/wordpress/?p=10979
		
		# rotate cw to facilitate sum function
		# rotated_ccw = zip(*original)[::-1]
		self.mcw = zip(*self.m[::-1])

		self.Efficiency = self.mcw[1][0] * 8 / (self.cap * self.time)
		self.Breakdown = math.fsum(self.mcw[1][self.ntcp+1:-1]) / self.mcw[1][0]
		#RE-BROKEN; nonsns; how many fucking bugs????
		#self.Breakdown = math.fsum(self.mcw[1][1:self.ntcp+1]) / self.mcw[1][0]

		
		self.Fairness.append( fairness(self.mcw[1][1:-1]) )
		self.Fairness.append( fairness(self.mcw[1][self.ntcp+1:-1]) )
		self.Fairness.append( fairness(self.mcw[1][1:self.ntcp+1]) )

		self.Droprate.append( self.mcw[2][0] )
		self.Droprate.append( math.fsum(self.mcw[2][self.ntcp+1:-1]) / self.ntcp )
		self.Droprate.append( math.fsum(self.mcw[2][1:self.ntcp+1]) / (len(self.mcw[2]) - 2 - self.ntcp) )

		self.CwndAvg.append( math.fsum(self.mcw[3][self.ntcp+1:-1]) / self.ntcp )
		self.CwndAvg.append( math.fsum(self.mcw[3][1:self.ntcp+1]) / (len(self.mcw[3]) - 2 - self.ntcp) )

		self.QueueAvg = self.mcw[4][0]

	def output(self):
		# use "print i," to ignore newline (for array)	
		# see http://docs.python.org/library/string.html
		#fms = '{0:8s} {1:8s} {2:8s} {3:8s} {4:8s} {5:8s} {6:8s} {7:8s} {8:8s} {9:8s} {10:8s}'
		#print fms.format('eta', 'tcp%',
		#				'fair_all', 'fair_tcp', 'fair_cc', 
		#				'drop_all', 'drop_tcp', 'drop_cc', 
		#				'cwnd_tcp', 'cwnd_cc', 'queue')
		print "eta tcp% fair_all fair_tcp fair_cc drop_all drop_tcp drop_cc cwnd_tcp cwnd_cc queue"
				

		#fmd = '{0:1.6f} {1:1.6f} {2:1.6f} {3:1.6f} {4:1.6f} {5:2.6f} {6:2.6f} {7:2.6f} {8:' '<8.2f} {9:' '<8.2f} {10:' '<8.2f}'
		fmd = '{0:1.6f} {1:1.6f} {2:1.6f} {3:1.6f} {4:1.6f} {5:2.6f} {6:2.6f} {7:.6f} {8:.2f} {9:.2f} {10:.2f}'
		print fmd.format(self.Efficiency, self.Breakdown, 
						self.Fairness[0], self.Fairness[1], self.Fairness[2],
						self.Droprate[0], self.Droprate[1], self.Droprate[2], 
						self.CwndAvg[0], self.CwndAvg[1], self.QueueAvg)


def main():
	parser = OptionParser()
	parser.add_option('-n', '--num-tcp', type='int', dest='ntcp', default=5, help='number of tcp flows')
	parser.add_option('-c', '--capacity', type='float', dest='cap', default=10e6, help='capacity of the network in Mbit/s')
	parser.add_option('-t', '--time', type='int', dest='time', default=60, help='time of simulation')
	opts, args = parser.parse_args()

	m = Metrics(opts.cap*1e6, opts.ntcp, opts.time)

	if len(args) < 1:
		print "usage: \n\n\
  		data-process.py [options] {datafiles} \n\
        -n	(=5)  number of files containing tcp flows \n\
        -c	(=10Mbps) capacity of the network \n"
		sys.exit()
	
	f = open(args[0], 'r')
	fdata = f.readlines()
	f.close()
	
	m.load(fdata)
	m.calculate()
	m.output()

if __name__ == '__main__':
	main()

